@@ -8,6 +8,8 @@ library(queuecomputer)
 
 base::load(file = "../20161017testdataframe.RData")
 
+#### queue_step
+
 set.seed(700)
 
 arrival_df <- data.frame(ID = c(1:100), times = rlnorm(100, meanlog = 3))
@@ -16,13 +18,15 @@ ord <- order(arrival_df$times)
 service <- rlnorm(100)
 server_list <- as.server.stepfun(c(15,30,50),c(1,3,1,10))
 
-firstqueue <- queue_step(arrival_df = arrival_df, service = service)
-secondqueue <- queue_step(arrival_df = arrival_df,
+firstqueue <- queue_step(arrival_df$times, service = service)
+secondqueue <- queue_step(arrival_df$times,
   servers = server_list, service = service)
 
 # testdataframe <- data.frame(arrival_df[ord, ], service[ord], firstqueue$times[ord], secondqueue$times[ord])
 
-newdataframe <- data.frame(arrival_df[ord, ], service[ord], firstqueue$times[ord], secondqueue$times[ord])
+newdataframe <- data.frame(arrival_df[ord, ], service[ord], firstqueue$departures_df$departures[ord], secondqueue$departures_df$departures[ord])
+
+names(newdataframe) <- c("ID", "times", "service.ord.", "firstqueue.times.ord.", "secondqueue.times.ord.")
 
 # save(testdataframe, file = "tests/20160921testdataframe.RData")
 
@@ -32,15 +36,17 @@ test_that("test that deterministic queue simulation departure times haven't chan
   expect_equal(newdataframe, testdataframe)
 })
 
+
 #Check lag_step and queue_step with large number of servers returns same results ------------------
 
-lag_queue <- lag_step(arrival_df = arrival_df, service = service)
+
+lag_queue <- arrival_df$times + service
 server_list <- as.server.stepfun(c(1),c(100,100))
 
-secondqueue <- queue_step(arrival_df = arrival_df, servers = server_list, service = service)
+secondqueue <- queue_step(arrival_df$times, servers = server_list, service = service)
 
 test_that("lag_step returns same results as queue_step with large number of servers", {
-  expect_equal(lag_queue$times, secondqueue$times)
+  expect_equal(lag_queue, secondqueue$departures_df$departures)
 })
 
 
@@ -54,11 +60,11 @@ ord <- order(arrival_df$times)
 
 ## Numeric -----------------------
 
-q1n <- queue_step(arrival_df, service, servers = 2)
-q2n <- queue_step(arrival_df[ord,], service[ord], servers = 2)
+q1n <- queue(arrival_df$times, service, servers = 2, serveroutput = TRUE)
+q2n <- queue(arrival_df$times[ord], service[ord], servers = 2, serveroutput = TRUE)
 
 test_that("reorder numeric", {
-  expect_equal(q1n$times, q2n[order(ord),]$times)
+  expect_equal(as.numeric(q1n), as.numeric(q2n)[order(ord)])
   expect_equal(attr(q1n, "server") ,attr(q2n, "server")[order(ord)])
 })
 
@@ -68,11 +74,11 @@ test_that("reorder numeric", {
 server_sf <- as.server.stepfun(c(50, 200),c(1,0,2))
 
 
-q1sf <- queue_step(arrival_df, service, servers = server_sf)
-q2sf <- queue_step(arrival_df[ord,], service[ord], servers = server_sf)
+q1sf <- queue_step(arrival_df$times, service, servers = server_sf)
+q2sf <- queue_step(arrival_df$times[ord], service[ord], servers = server_sf)
 
 test_that("reorder stepfun", {
-  expect_equal(q1sf$times , q2sf[order(ord),]$times)
+  expect_equal(q1sf$departures_df , q2sf$departures_df[order(ord),])
 })
 
 ## server.list -------------------------
@@ -80,11 +86,11 @@ test_that("reorder stepfun", {
 server_list <- queuecomputer:::server_make(c(50, 200),c(1,0,2))
 
 
-q1sl <- queue_step(arrival_df, service, servers = server_list)
-q2sl <- queue_step(arrival_df[ord,], service[ord], servers = server_list)
+q1sl <- queue_step(arrival_df$times, service, servers = server_list)
+q2sl <- queue_step(arrival_df$times[ord], service[ord], servers = server_list)
 
 test_that("reorder server.list", {
-  expect_equal(q1sl$times , q2sl[order(ord),]$times)
+  expect_equal(q1sl$departures_df , q2sl$departures_df[order(ord),])
 })
 
 ## Check summaries don't return errors ----------------
@@ -104,11 +110,11 @@ ord <- order(arrival_df$times)
 server_sf <- as.server.stepfun(c(50, 200, 250, 275),c(1,2,1,3,1))
 server_list <- queuecomputer:::server_make(c(50, 200, 250, 275),c(1,2,1,3,1))
 
-qsf <- queue_step(arrival_df, service, servers = server_sf)
-qsl <- queue_step(arrival_df, service, servers = server_list)
+qsf <- queue_step(arrival_df$times, service, servers = server_sf)
+qsl <- queue_step(arrival_df$times, service, servers = server_list)
 
 test_that("Check service times", {
-  expect_equal(qsf$times , qsl$times)
+  expect_equal(qsf$departures_df , qsl$departures_df)
 })
 
 service <- pmin.int(rlnorm(500, meanlog = 3), 24)
@@ -116,21 +122,21 @@ service <- pmin.int(rlnorm(500, meanlog = 3), 24)
 server_sf <- as.server.stepfun(c(50, 200, 250, 275),c(1,0,1,0,1))
 server_list <- queuecomputer:::server_make(c(50, 200, 250, 275),c(1,0,1,0,1))
 
-qsf <- queue_step(arrival_df, service, servers = server_sf)
-qsl <- queue_step(arrival_df, service, servers = server_list)
+qsf <- queue_step(arrival_df$times, service, servers = server_sf)
+qsl <- queue_step(arrival_df$times, service, servers = server_list)
 
 test_that("Check service times, zeros", {
-  expect_equal(qsf$times , qsl$times)
+  expect_equal(qsf$departures_df , qsl$departures_df)
 })
 
 server_sf <- as.server.stepfun(c(50, 200, 250, 275),c(1,0,1,0,1))
 server_list <- as.server.list(list(c(50, 200, 250, 275)),1)
 
-qsf <- queue_step(arrival_df, service, servers = server_sf)
-qsl <- queue_step(arrival_df, service, servers = server_list)
+qsf <- queue_step(arrival_df$times, service, servers = server_sf)
+qsl <- queue_step(arrival_df$times, service, servers = server_list)
 
 test_that("Check service times, zeros with as.server.list", {
-    expect_equal(qsf$times , qsl$times)
+    expect_equal(qsf$departures_df , qsl$departures_df)
 })
 
 # Check that a warning is produced
@@ -142,7 +148,7 @@ service <- rlnorm(500, meanlog = 3)
 server_sf <- as.server.stepfun(c(50, 200, 250, 275),c(1,0,1,0,1))
 
 test_that("Check warning is produced", {
-  expect_warning(qsf <- queue_step(arrival_df, service, servers = server_sf))
+  expect_warning(qsf <- queue_step(arrival_df$times, service, servers = server_sf))
 })
 
 
@@ -157,10 +163,11 @@ ord <- order(arrival_df$times)
 
 server_sf <- as.server.stepfun(c(50), c(2,2))
 
-qsf <- queue_step(arrival_df, service, servers = server_sf)
-qn <- queue_step(arrival_df, service, servers = 2)
+qsf <- queue_step(arrival_df$times, service, servers = server_sf)
+qn <- queue_step(arrival_df$times, service, servers = 2)
 
-all(qsf == qn)
+test_that("Check queue_step.server.stepfun and queue_step.numeric", { expect_equal(qsf$departures_df, qn$departures_df)
+})
 
 # Check second queue output is later than first --------------
 
@@ -170,22 +177,17 @@ arrival_df <- data.frame(ID = c(1:n_customers), times = rlnorm(n_customers, mean
 service_1 <- rlnorm(n_customers)
 
 
-firstqueue <- queue_step(arrival_df = arrival_df,
+firstqueue <- queue_step(arrival_df$times,
   servers = 2, service = service_1)
 
 server_list <- as.server.stepfun(c(50),c(1,2))
 
 service_2 <- rlnorm(n_customers)
-secondqueue <- queue_step(arrival_df = firstqueue,
+secondqueue <- queue_step(firstqueue,
   servers = server_list, service = service_2)
 
-all(firstqueue$times >= secondqueue$times)
-
-
-# Wait step
-
-test_that("Wait step for bags", {
-    expect_equal(wait_step(arrival_df, service)$times, pmax.int(arrival_df$times, service))
+expect_true(info = "second queue later than first", {
+  all(firstqueue$departures_df$departures <= secondqueue$departures_df$departures)
 })
 
 
